# -*- coding: utf-8 -*-

import unittest

from steenzout.object import Object


class ObjectTestCase(unittest.TestCase):
    """Test case for the Object class."""

    def test_hash(self):
        """Test for the __hash__ function."""

        o1 = Object()
        o2 = Object()
        a = object()

        self.assertEqual(hash(o1), hash(o2))
        self.assertNotEqual(hash(o1), hash(a))

    def test_eq(self):
        """Test for the __eq__ function."""

        o1 = Object()
        o2 = Object()
        a = object()

        self.assertTrue(o1 == o1)
        self.assertTrue(o1 == o2)

        self.assertFalse(id(o1) == id(o2))
        self.assertFalse(o1 == a)

    def test_neq(self):
        """Test for the __neq__ function."""

        o1 = Object()
        o2 = Object()
        a = object()

        self.assertFalse(o1 != o1)
        self.assertFalse(o1 != o2)

        self.assertTrue(o1 != a)

    def test_repr(self):
        """Test for the __repr__ function."""
        o1 = Object()

        self.assertEqual("<class 'steenzout.object.Object'>({})", repr(o1))

    def test_str(self):
        """Test for the __str__ function."""
        o1 = Object()
        self.assertEqual(str(o1), 'steenzout.object.Object')

        class Mock(Object):
            def __init__(self):
                self.x = 1
                self.y = 'a'
                self.l = ['a']
                self.d = {'a': 1}
                self.b = True

        m1 = Mock()
        assert str(m1) == 'tests.object_test.Mock b=True d={\'a\': 1} l=[\'a\'] x=1 y=a'



# -*- coding: utf-8 -*-

import unittest

from six import with_metaclass
from steenzout.object import Object, Singleton


class MockSingleton1(Object, with_metaclass(Singleton)):

    def __init__(self, value):
        super(MockSingleton1, self).__init__()
        self.attr = value


class MockSingleton2(Object, with_metaclass(Singleton)):

    def __init__(self, value):
        super(MockSingleton2, self).__init__()
        self.attr = value


class SingletonTestCase(unittest.TestCase):
    """Test case for the Singleton class."""

    def test(self):
        x = MockSingleton1(1)
        y = MockSingleton1(1)

        self.assertEqual(1, x.attr)
        self.assertEqual(1, y.attr)
        self.assertEqual(x, y)
        self.assertEqual(id(x), id(y))

        w = MockSingleton2(1)
        z = MockSingleton2(2)

        self.assertEqual(1, w.attr)
        self.assertEqual(1, z.attr)
        self.assertEqual(w, z)
        self.assertEqual(id(w), id(z))
        self.assertNotEqual(id(w), id(x))

        z.attr = 2

        self.assertEqual(2, z.attr)
        self.assertEqual(2, w.attr)



# -*- coding: utf-8 -*-
#
# Copyright 2016 Pedro Salgado
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Metadata module."""

__author__ = 'Pedro Salgado'
__author_email__ = 'steenzout@ymail.com'

__classifiers__ = (
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Natural Language :: English',
    'License :: OSI Approved :: Apache Software License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: Implementation :: CPython',
    'Programming Language :: Python :: Implementation :: PyPy'
)

__description__ = 'Steenzout Python objects.'

__license__ = 'Apache 2.0'

__maintainer__ = 'Pedro Salgado'
__maintainer_email__ = 'steenzout@ymail.com'

__url__ = 'https://github.com/steenzout/python-object/'

__version__ = '1.0.12'



# -*- coding: utf-8 -*-
#
# Copyright 2016 Pedro Salgado
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Object module."""

import itertools
import threading


from steenzout.object.metadata import __version__


class Object(object):
    """Object class with a `__hash__`, `__eq__`, `__ne__` and `__repr__` implementation."""

    def __hash__(self):
        """Called by built-in function :function:`object.__hash__`.

        See :py:function:`object.__hash__`.

        Returns:
            (integer): hash value.
        """
        return hash(tuple(sorted(self.__dict__.items())))

    def __eq__(self, other):
        """Rich comparison function for the `==` operator.

        Args:
            other (:py:class:`Object`): instance to be compared against.

        Raises:
            :py:class:`exceptions.NotImplementedError` when
                the `other` instance is not a `Object` object.

        Returns:
            (bool): True in case both instances represent the same object;
                False otherwise.
        """
        if other is self:
            return True
        elif isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        else:
            return NotImplemented

    def __ne__(self, other):
        """Rich comparison function for the `!=` operator.

        Args:
            other (:py:class:`Object`): instance to be compared against.

        Raises:
            :class:`exceptions.NotImplementedError` when
                the `other` instance is not a `Object` object.

        Returns:
            (bool) True in case `self` and `other`
            do not represent the same object; False otherwise.
        """
        if isinstance(other, self.__class__):
            return not self == other
        return NotImplemented

    def __repr__(self):
        """Return the “official” string representation of this object.

        Returns:
            (str): the “official” string representation of this object.
        """
        return '%s(%r)' % (self.__class__, self.__dict__)

    def __str__(self):
        """Returns the "informal" string representation of this object

        Returns:
            (str): the "informal" string representation of this object.
        """
        return ' '.join(itertools.chain(
            ('%s.%s' % (self.__class__.__module__, self.__class__.__name__),),
            ('%s=%s' % x for x in sorted(self.__dict__.items()))))


class Singleton(type):
    """Singleton metaclass."""

    instance = None

    def __call__(cls, *args, **kwargs):
        """Return singleton instance.

        Args:
            cls (type): the class.
            args (tuple/list): initializer function arguments.
            kwargs (dict): initializer function keyword arguments.
        """
        if cls.instance is None:
            with threading.Lock():
                if cls.instance is None:
                    cls.instance = super(Singleton, cls).__call__(*args, **kwargs)

        return cls.instance


def version():
    """Return this package version.

    Returns:
        (str): package version.
    """
    return __version__



# -*- coding: utf-8 -*-

import unittest

import steenzout.object


class AttributesTestCase(unittest.TestCase):
    """steenzout.object package test cases."""

    def test_version(self):
        """Test version function."""

        self.assertTrue(
            steenzout.object.version() == steenzout.object.__version__
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pip.download

from pip.req import parse_requirements

from setuptools import find_packages, setup

exec(open('steenzout/object/metadata.py').read())

setup(name='steenzout.object',
      version=__version__,
      description=__description__,
      author=__author__,
      author_email=__author_email__,
      maintainer=__maintainer__,
      maintainer_email=__maintainer_email__,
      url=__url__,
      namespace_packages=['steenzout'],
      packages=find_packages(exclude=('*.tests', '*.tests.*', 'tests.*', 'tests')),
      package_data={'': ['LICENSE', 'NOTICE.md']},
      install_requires=[
          str(pkg.req) for pkg in parse_requirements(
              'requirements.txt', session=pip.download.PipSession())],
      tests_require=[
          str(pkg.req) for pkg in parse_requirements(
              'requirements-test.txt', session=pip.download.PipSession())],
      license=__license__,
      classifiers=__classifiers__)
